@@ -17,27 +17,37 @@
 #  include "libfswatch_config.h"
 #endif
 
-#include "libfswatch_mem.h"
 #include "cevent.h"
-#include "../c++/event.hpp"
+#include <cstdlib>
 #include <cstring>
+#include "../c++/event.hpp"
+#include "../c++/libfswatch_exception.hpp"
 
 using namespace std;
 using namespace fsw;
 
-fsw_event_flag fsw_get_event_flag_by_name(const char * name)
+FSW_STATUS fsw_get_event_flag_by_name(const char *name, fsw_event_flag *flag)
 {
-  return event::get_event_flag_by_name(name);
+  try
+  {
+    *flag = event::get_event_flag_by_name(name);
+
+    return FSW_OK;
+  }
+  catch (const libfsw_exception& ex)
+  {
+    return FSW_ERR_UNKNOWN_VALUE;
+  }
 }
 
-char * fsw_get_event_flag_name(const fsw_event_flag flag)
+char *fsw_get_event_flag_name(const fsw_event_flag flag)
 {
   string name = event::get_event_flag_name(flag);
-  char * cstr = static_cast<char *>(fsw_alloc(name.size() + 1));
-  
+  char *cstr = static_cast<char *>(malloc(name.size() + 1));
+
   if (cstr == nullptr) return nullptr;
-  
+
   strcpy(cstr, name.c_str());
-  
+
   return cstr;
 }
